using System;
using System.Web.Script.Serialization;
using System.Xml;
using System.Collections.Generic;

using MindFusion.Diagramming;
using MindFusion.Diagramming.WebForms;


public class CustomNode : ShapeNode
{
    public CustomNode()
    {
        nodeType = 1;
        label = "";
    }

    protected override void SaveToXml(XmlElement xmlElement, XmlPersistContext context)
    {
        base.SaveToXml(xmlElement, context);
        context.WriteInt(nodeType, "NodeType", xmlElement);
        context.WriteString(label, "Label", xmlElement);
    }

    protected override void LoadFromXml(XmlElement xmlElement, XmlPersistContext context)
    {
        base.LoadFromXml(xmlElement, context);
        nodeType = context.ReadInt("NodeType", xmlElement);
        label = context.ReadString("Label", xmlElement);
    }

    protected override void SaveTo(System.IO.BinaryWriter writer, PersistContext context)
    {
        base.SaveTo(writer, context);
        writer.Write(nodeType);
        writer.Write(label);
    }

    protected override void LoadFrom(System.IO.BinaryReader reader, PersistContext context)
    {
        base.LoadFrom(reader, context);
        nodeType = reader.ReadInt16();
        label = reader.ReadString();
    }


    public int NodeType
    {
        get { return nodeType; }
        set
        {
            nodeType = value;
        }
    }

    public string Label
    {
        get { return label; }
        set
        {
            label = value;
        }
    }


    private int nodeType;
    private string label;
}


public class CustomNodeConverter : ShapeNodeConverter
{
    public CustomNodeConverter(DiagramView view)
        : base(view)
    {
    }

    public override IEnumerable<Type> SupportedTypes
    {
        get { return new List<Type>(new Type[] { typeof(CustomNode) }); }
    }

    public override object Deserialize(
		IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
    {
        CustomNode node = (CustomNode)base.Deserialize(dictionary, type, serializer);
        node.NodeType = int.Parse(dictionary["nodeType"].ToString());
        node.Label = dictionary["label"].ToString();
        return node;
    }

    public override IDictionary<string, object> Serialize(
		object obj, JavaScriptSerializer serializer)
    {
        CustomNode node = obj as CustomNode;
        if (node != null)
        {
            IDictionary<string, object> json = base.Serialize(obj, serializer);
            if (json != null)
            {
                json.Add("nodeType", node.NodeType);
                json.Add("label", node.Label);
            }
            return json;
        }

        return null;
    }
}