using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml;
using System.Web.Script.Serialization;

using MindFusion.Diagramming;
using MindFusion.Diagramming.WebForms;
using MindFusion.Drawing;


public class IconLink : DiagramLink
{
	public IconLink(Diagram diagram)
		: base(diagram)
	{
		midPointShape = PointShape.Circle;
	}

	public IconLink(IconLink prototype, DiagramNode src, DiagramNode dest)
		: base(prototype, src, dest)
	{
		midPointShape = prototype.midPointShape;
	}

	public override DiagramItem Clone(bool clipboard)
	{
		return new IconLink(this, GetDummyNode(), GetDummyNode());
	}

	internal DummyNode GetDummyNode()
	{
		if (Parent != null)
			return Parent.Dummy;

		return new DummyNode(null);
	}

	public override void DrawShadow(IGraphics graphics, RenderOptions options)
	{
	}

	public override void Draw(IGraphics graphics, RenderOptions options)
	{

		System.Drawing.Pen pen = new System.Drawing.Pen(Color.Black, 0.1f);
		System.Drawing.Brush brush = new System.Drawing.SolidBrush(Color.Black);

		PointCollection cp = this.ControlPoints;
		GraphicsPath path = new GraphicsPath();

		path.AddLine(cp[0], cp[1]);

		PointF p0 = new PointF((cp[0].X + cp[1].X) / 2, (cp[0].Y + cp[1].Y) / 2);
		PointF p1 = new PointF((cp[0].X + p0.X) / 2, (cp[0].Y + p0.Y) / 2);
		PointF p2 = new PointF((p0.X + cp[1].X) / 2, (p0.Y + cp[1].Y) / 2);

		switch (midPointShape)
		{
			case PointShape.Circle:
				{
					brush = new System.Drawing.SolidBrush(Color.CornflowerBlue);
					path.AddEllipse(p0.X - 1, p0.Y - 1, 2, 2);
					path.AddEllipse(p1.X - 1, p1.Y - 1, 2, 2);
					path.AddEllipse(p2.X - 1, p2.Y - 1, 2, 2);
					break;
				}
			case PointShape.Rectangle:
				{
					brush = new System.Drawing.SolidBrush(Color.PaleGoldenrod);
					path.AddRectangle(new RectangleF(p0.X - 1, p0.Y - 1, 2, 2));
					path.AddRectangle(new RectangleF(p1.X - 1, p1.Y - 1, 2, 2));
					path.AddRectangle(new RectangleF(p2.X - 1, p2.Y - 1, 2, 2));
					break;
				}
		}

		graphics.DrawPath(pen, path);
		graphics.FillPath(brush, path);
		path.Dispose();
	}

	protected override void SaveToXml(XmlElement xmlElement, XmlPersistContext context)
	{
		base.SaveToXml(xmlElement, context);
		context.WriteEnum(midPointShape, "MidPointShape", xmlElement);
	}

	protected override void LoadFromXml(XmlElement xmlElement, XmlPersistContext context)
	{
		base.LoadFromXml(xmlElement, context);
		midPointShape = (PointShape)context.ReadEnum("MidPointShape", xmlElement);
	}

	protected override void SaveTo(System.IO.BinaryWriter writer, PersistContext context)
	{
		base.SaveTo(writer, context);
		writer.Write((int)midPointShape);
	}

	protected override void LoadFrom(System.IO.BinaryReader reader, PersistContext context)
	{
		base.LoadFrom(reader, context);
		midPointShape = (PointShape)reader.ReadInt32();
	}

	public PointShape MidPointShape
	{
		get { return midPointShape; }
		set
		{
			midPointShape = value;
		}
	}

	private PointShape midPointShape;
	public enum PointShape
	{
		Circle,
		Rectangle
	}
}

public class IconLinkConverter : DiagramLinkConverter
{
    public IconLinkConverter(DiagramView view)
        : base(view, true)
    {
    }

    public override IEnumerable<Type> SupportedTypes
    {
        get { return new List<Type>(new Type[] { typeof(IconLink) }); }
    }

    public override object Deserialize(
		IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
    {
		IconLink link = (IconLink)base.Deserialize(dictionary, type, serializer);
        link.MidPointShape = (IconLink.PointShape)dictionary["midPointShape"];
        return link;
    }

    public override IDictionary<string, object> Serialize(
		object obj, JavaScriptSerializer serializer)
    {
        IconLink link = obj as IconLink;
        if (link != null)
        {
            IDictionary<string, object> json = base.Serialize(obj, serializer);
            if (json != null)
            {
                json.Add("midPointShape", link.MidPointShape);
            }
            return json;
        }

        return null;
    }
}